@@ -9,6 +9,7 @@ import shutil
 import subprocess
 import tempfile
 import webbrowser
+from datetime import datetime
 from pathlib import Path
 
 import click
@@ -47,6 +48,33 @@ LONG_TEXT_THRESHOLD = (
     300  # Characters - text blocks longer than this are shown in index
 )
 
+
+def extract_text_from_content(content):
+    """Extract plain text from message content.
+
+    Handles both string content (older format) and array content (newer format).
+
+    Args:
+        content: Either a string or a list of content blocks like
+                 [{"type": "text", "text": "..."}, {"type": "image", ...}]
+
+    Returns:
+        The extracted text as a string, or empty string if no text found.
+    """
+    if isinstance(content, str):
+        return content.strip()
+    elif isinstance(content, list):
+        # Extract text from content blocks of type "text"
+        texts = []
+        for block in content:
+            if isinstance(block, dict) and block.get("type") == "text":
+                text = block.get("text", "")
+                if text:
+                    texts.append(text)
+        return " ".join(texts).strip()
+    return ""
+
+
 # Module-level variable for GitHub repo (set by generate_html)
 _github_repo = None
 
@@ -74,10 +102,11 @@ def get_session_summary(filepath, max_length=200):
                 if entry.get("type") == "user":
                     msg = entry.get("message", {})
                     content = msg.get("content", "")
-                    if isinstance(content, str) and content.strip():
-                        if len(content) > max_length:
-                            return content[: max_length - 3] + "..."
-                        return content
+                    text = extract_text_from_content(content)
+                    if text:
+                        if len(text) > max_length:
+                            return text[: max_length - 3] + "..."
+                        return text
             return "(no summary)"
     except Exception:
         return "(no summary)"
@@ -116,12 +145,11 @@ def _get_jsonl_summary(filepath, max_length=200):
                         and obj.get("message", {}).get("content")
                     ):
                         content = obj["message"]["content"]
-                        if isinstance(content, str):
-                            content = content.strip()
-                            if content and not content.startswith("<"):
-                                if len(content) > max_length:
-                                    return content[: max_length - 3] + "..."
-                                return content
+                        text = extract_text_from_content(content)
+                        if text and not text.startswith("<"):
+                            if len(text) > max_length:
+                                return text[: max_length - 3] + "..."
+                            return text
                 except json.JSONDecodeError:
                     continue
     except Exception:
@@ -155,6 +183,271 @@ def find_local_sessions(folder, limit=10):
     return results[:limit]
 
 
+def get_project_display_name(folder_name):
+    """Convert encoded folder name to readable project name.
+
+    Claude Code stores projects in folders like:
+    - -home-user-projects-myproject -> myproject
+    - -mnt-c-Users-name-Projects-app -> app
+
+    For nested paths under common roots (home, projects, code, Users, etc.),
+    extracts the meaningful project portion.
+    """
+    # Common path prefixes to strip
+    prefixes_to_strip = [
+        "-home-",
+        "-mnt-c-Users-",
+        "-mnt-c-users-",
+        "-Users-",
+    ]
+
+    name = folder_name
+    for prefix in prefixes_to_strip:
+        if name.lower().startswith(prefix.lower()):
+            name = name[len(prefix) :]
+            break
+
+    # Split on dashes and find meaningful parts
+    parts = name.split("-")
+
+    # Common intermediate directories to skip
+    skip_dirs = {"projects", "code", "repos", "src", "dev", "work", "documents"}
+
+    # Find the first meaningful part (after skipping username and common dirs)
+    meaningful_parts = []
+    found_project = False
+
+    for i, part in enumerate(parts):
+        if not part:
+            continue
+        # Skip the first part if it looks like a username (before common dirs)
+        if i == 0 and not found_project:
+            # Check if next parts contain common dirs
+            remaining = [p.lower() for p in parts[i + 1 :]]
+            if any(d in remaining for d in skip_dirs):
+                continue
+        if part.lower() in skip_dirs:
+            found_project = True
+            continue
+        meaningful_parts.append(part)
+        found_project = True
+
+    if meaningful_parts:
+        return "-".join(meaningful_parts)
+
+    # Fallback: return last non-empty part or original
+    for part in reversed(parts):
+        if part:
+            return part
+    return folder_name
+
+
+def find_all_sessions(folder, include_agents=False):
+    """Find all sessions in a Claude projects folder, grouped by project.
+
+    Returns a list of project dicts, each containing:
+    - name: display name for the project
+    - path: Path to the project folder
+    - sessions: list of session dicts with path, summary, mtime, size
+
+    Sessions are sorted by modification time (most recent first) within each project.
+    Projects are sorted by their most recent session.
+    """
+    folder = Path(folder)
+    if not folder.exists():
+        return []
+
+    projects = {}
+
+    for session_file in folder.glob("**/*.jsonl"):
+        # Skip agent files unless requested
+        if not include_agents and session_file.name.startswith("agent-"):
+            continue
+
+        # Get summary and skip boring sessions
+        summary = get_session_summary(session_file)
+        if summary.lower() == "warmup" or summary == "(no summary)":
+            continue
+
+        # Get project folder
+        project_folder = session_file.parent
+        project_key = project_folder.name
+
+        if project_key not in projects:
+            projects[project_key] = {
+                "name": get_project_display_name(project_key),
+                "path": project_folder,
+                "sessions": [],
+            }
+
+        stat = session_file.stat()
+        projects[project_key]["sessions"].append(
+            {
+                "path": session_file,
+                "summary": summary,
+                "mtime": stat.st_mtime,
+                "size": stat.st_size,
+            }
+        )
+
+    # Sort sessions within each project by mtime (most recent first)
+    for project in projects.values():
+        project["sessions"].sort(key=lambda s: s["mtime"], reverse=True)
+
+    # Convert to list and sort projects by most recent session
+    result = list(projects.values())
+    result.sort(
+        key=lambda p: p["sessions"][0]["mtime"] if p["sessions"] else 0, reverse=True
+    )
+
+    return result
+
+
+def generate_batch_html(
+    source_folder, output_dir, include_agents=False, progress_callback=None
+):
+    """Generate HTML archive for all sessions in a Claude projects folder.
+
+    Creates:
+    - Master index.html listing all projects
+    - Per-project directories with index.html listing sessions
+    - Per-session directories with transcript pages
+
+    Args:
+        source_folder: Path to the Claude projects folder
+        output_dir: Path for output archive
+        include_agents: Whether to include agent-* session files
+        progress_callback: Optional callback(project_name, session_name, current, total)
+            called after each session is processed
+
+    Returns statistics dict with total_projects, total_sessions, failed_sessions, output_dir.
+    """
+    source_folder = Path(source_folder)
+    output_dir = Path(output_dir)
+    output_dir.mkdir(parents=True, exist_ok=True)
+
+    # Find all sessions
+    projects = find_all_sessions(source_folder, include_agents=include_agents)
+
+    # Calculate total for progress tracking
+    total_session_count = sum(len(p["sessions"]) for p in projects)
+    processed_count = 0
+    successful_sessions = 0
+    failed_sessions = []
+
+    # Process each project
+    for project in projects:
+        project_dir = output_dir / project["name"]
+        project_dir.mkdir(exist_ok=True)
+
+        # Process each session
+        for session in project["sessions"]:
+            session_name = session["path"].stem
+            session_dir = project_dir / session_name
+
+            # Generate transcript HTML with error handling
+            try:
+                generate_html(session["path"], session_dir)
+                successful_sessions += 1
+            except Exception as e:
+                failed_sessions.append(
+                    {
+                        "project": project["name"],
+                        "session": session_name,
+                        "error": str(e),
+                    }
+                )
+
+            processed_count += 1
+
+            # Call progress callback if provided
+            if progress_callback:
+                progress_callback(
+                    project["name"], session_name, processed_count, total_session_count
+                )
+
+        # Generate project index
+        _generate_project_index(project, project_dir)
+
+    # Generate master index
+    _generate_master_index(projects, output_dir)
+
+    return {
+        "total_projects": len(projects),
+        "total_sessions": successful_sessions,
+        "failed_sessions": failed_sessions,
+        "output_dir": output_dir,
+    }
+
+
+def _generate_project_index(project, output_dir):
+    """Generate index.html for a single project."""
+    template = get_template("project_index.html")
+
+    # Format sessions for template
+    sessions_data = []
+    for session in project["sessions"]:
+        mod_time = datetime.fromtimestamp(session["mtime"])
+        sessions_data.append(
+            {
+                "name": session["path"].stem,
+                "summary": session["summary"],
+                "date": mod_time.strftime("%Y-%m-%d %H:%M"),
+                "size_kb": session["size"] / 1024,
+            }
+        )
+
+    html_content = template.render(
+        project_name=project["name"],
+        sessions=sessions_data,
+        session_count=len(sessions_data),
+        css=CSS,
+        js=JS,
+    )
+
+    output_path = output_dir / "index.html"
+    output_path.write_text(html_content, encoding="utf-8")
+
+
+def _generate_master_index(projects, output_dir):
+    """Generate master index.html listing all projects."""
+    template = get_template("master_index.html")
+
+    # Format projects for template
+    projects_data = []
+    total_sessions = 0
+
+    for project in projects:
+        session_count = len(project["sessions"])
+        total_sessions += session_count
+
+        # Get most recent session date
+        if project["sessions"]:
+            most_recent = datetime.fromtimestamp(project["sessions"][0]["mtime"])
+            recent_date = most_recent.strftime("%Y-%m-%d")
+        else:
+            recent_date = "N/A"
+
+        projects_data.append(
+            {
+                "name": project["name"],
+                "session_count": session_count,
+                "recent_date": recent_date,
+            }
+        )
+
+    html_content = template.render(
+        projects=projects_data,
+        total_projects=len(projects),
+        total_sessions=total_sessions,
+        css=CSS,
+        js=JS,
+    )
+
+    output_path = output_dir / "index.html"
+    output_path.write_text(html_content, encoding="utf-8")
+
+
 def parse_session_file(filepath):
     """Parse a session file and return normalized data.
 
@@ -385,7 +678,12 @@ def render_content_block(block):
     if not isinstance(block, dict):
         return f"<p>{html.escape(str(block))}</p>"
     block_type = block.get("type", "")
-    if block_type == "thinking":
+    if block_type == "image":
+        source = block.get("source", {})
+        media_type = source.get("media_type", "image/png")
+        data = source.get("data", "")
+        return _macros.image_block(media_type, data)
+    elif block_type == "thinking":
         content_html = render_markdown_text(block.get("thinking", ""))
         return _macros.thinking(content_html)
     elif block_type == "text":
@@ -585,6 +883,8 @@ CSS = """
 body { font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif; background: var(--bg-color); color: var(--text-color); margin: 0; padding: 16px; line-height: 1.6; }
 .container { max-width: 800px; margin: 0 auto; }
 h1 { font-size: 1.5rem; margin-bottom: 24px; padding-bottom: 8px; border-bottom: 2px solid var(--user-border); }
+.header-row { display: flex; justify-content: space-between; align-items: center; flex-wrap: wrap; gap: 12px; border-bottom: 2px solid var(--user-border); padding-bottom: 8px; margin-bottom: 24px; }
+.header-row h1 { border-bottom: none; padding-bottom: 0; margin-bottom: 0; flex: 1; min-width: 200px; }
 .message { margin-bottom: 16px; border-radius: 12px; overflow: hidden; box-shadow: 0 1px 3px rgba(0,0,0,0.1); }
 .message.user { background: var(--user-bg); border-left: 4px solid var(--user-border); }
 .message.assistant { background: var(--card-bg); border-left: 4px solid var(--assistant-border); }
@@ -695,7 +995,25 @@ details.continuation[open] summary { border-radius: 12px 12px 0 0; margin-bottom
 .index-item-long-text { margin-top: 8px; padding: 12px; background: var(--card-bg); border-radius: 8px; border-left: 3px solid var(--assistant-border); }
 .index-item-long-text .truncatable.truncated::after { background: linear-gradient(to bottom, transparent, var(--card-bg)); }
 .index-item-long-text-content { color: var(--text-color); }
-@media (max-width: 600px) { body { padding: 8px; } .message, .index-item { border-radius: 8px; } .message-content, .index-item-content { padding: 12px; } pre { font-size: 0.8rem; padding: 8px; } }
+#search-box { display: none; align-items: center; gap: 8px; }
+#search-box input { padding: 6px 12px; border: 1px solid var(--assistant-border); border-radius: 6px; font-size: 16px; width: 180px; }
+#search-box button, #modal-search-btn, #modal-close-btn { background: var(--user-border); color: white; border: none; border-radius: 6px; padding: 6px 10px; cursor: pointer; display: flex; align-items: center; justify-content: center; }
+#search-box button:hover, #modal-search-btn:hover { background: #1565c0; }
+#modal-close-btn { background: var(--text-muted); margin-left: 8px; }
+#modal-close-btn:hover { background: #616161; }
+#search-modal[open] { border: none; border-radius: 12px; box-shadow: 0 4px 24px rgba(0,0,0,0.2); padding: 0; width: 90vw; max-width: 900px; height: 80vh; max-height: 80vh; display: flex; flex-direction: column; }
+#search-modal::backdrop { background: rgba(0,0,0,0.5); }
+.search-modal-header { display: flex; align-items: center; gap: 8px; padding: 16px; border-bottom: 1px solid var(--assistant-border); background: var(--bg-color); border-radius: 12px 12px 0 0; }
+.search-modal-header input { flex: 1; padding: 8px 12px; border: 1px solid var(--assistant-border); border-radius: 6px; font-size: 16px; }
+#search-status { padding: 8px 16px; font-size: 0.85rem; color: var(--text-muted); border-bottom: 1px solid rgba(0,0,0,0.06); }
+#search-results { flex: 1; overflow-y: auto; padding: 16px; }
+.search-result { margin-bottom: 16px; border-radius: 8px; overflow: hidden; box-shadow: 0 1px 3px rgba(0,0,0,0.1); }
+.search-result a { display: block; text-decoration: none; color: inherit; }
+.search-result a:hover { background: rgba(25, 118, 210, 0.05); }
+.search-result-page { padding: 6px 12px; background: rgba(0,0,0,0.03); font-size: 0.8rem; color: var(--text-muted); border-bottom: 1px solid rgba(0,0,0,0.06); }
+.search-result-content { padding: 12px; }
+.search-result mark { background: #fff59d; padding: 1px 2px; border-radius: 2px; }
+@media (max-width: 600px) { body { padding: 8px; } .message, .index-item { border-radius: 8px; } .message-content, .index-item-content { padding: 12px; } pre { font-size: 0.8rem; padding: 8px; } #search-box input { width: 120px; } #search-modal[open] { width: 95vw; height: 90vh; } }
 """
 
 JS = """
@@ -729,24 +1047,96 @@ document.querySelectorAll('.truncatable').forEach(function(wrapper) {
 });
 """
 
-# JavaScript to fix relative URLs when served via gistpreview.github.io
+# JavaScript to fix relative URLs when served via gisthost.github.io or gistpreview.github.io
+# Fixes issue #26: Pagination links broken on gisthost.github.io
 GIST_PREVIEW_JS = r"""
 (function() {
-    if (window.location.hostname !== 'gistpreview.github.io') return;
-    // URL format: https://gistpreview.github.io/?GIST_ID/filename.html
+    var hostname = window.location.hostname;
+    if (hostname !== 'gisthost.github.io' && hostname !== 'gistpreview.github.io') return;
+    // URL format: https://gisthost.github.io/?GIST_ID/filename.html
     var match = window.location.search.match(/^\?([^/]+)/);
     if (!match) return;
     var gistId = match[1];
-    document.querySelectorAll('a[href]').forEach(function(link) {
-        var href = link.getAttribute('href');
-        // Skip external links and anchors
-        if (href.startsWith('http') || href.startsWith('#') || href.startsWith('//')) return;
-        // Handle anchor in relative URL (e.g., page-001.html#msg-123)
-        var parts = href.split('#');
-        var filename = parts[0];
-        var anchor = parts.length > 1 ? '#' + parts[1] : '';
-        link.setAttribute('href', '?' + gistId + '/' + filename + anchor);
+
+    function rewriteLinks(root) {
+        (root || document).querySelectorAll('a[href]').forEach(function(link) {
+            var href = link.getAttribute('href');
+            // Skip already-rewritten links (issue #26 fix)
+            if (href.startsWith('?')) return;
+            // Skip external links and anchors
+            if (href.startsWith('http') || href.startsWith('#') || href.startsWith('//')) return;
+            // Handle anchor in relative URL (e.g., page-001.html#msg-123)
+            var parts = href.split('#');
+            var filename = parts[0];
+            var anchor = parts.length > 1 ? '#' + parts[1] : '';
+            link.setAttribute('href', '?' + gistId + '/' + filename + anchor);
+        });
+    }
+
+    // Run immediately
+    rewriteLinks();
+
+    // Also run on DOMContentLoaded in case DOM isn't ready yet
+    if (document.readyState === 'loading') {
+        document.addEventListener('DOMContentLoaded', function() { rewriteLinks(); });
+    }
+
+    // Use MutationObserver to catch dynamically added content
+    // gistpreview.github.io may add content after initial load
+    var observer = new MutationObserver(function(mutations) {
+        mutations.forEach(function(mutation) {
+            mutation.addedNodes.forEach(function(node) {
+                if (node.nodeType === 1) { // Element node
+                    rewriteLinks(node);
+                    // Also check if the node itself is a link
+                    if (node.tagName === 'A' && node.getAttribute('href')) {
+                        var href = node.getAttribute('href');
+                        if (!href.startsWith('?') && !href.startsWith('http') &&
+                            !href.startsWith('#') && !href.startsWith('//')) {
+                            var parts = href.split('#');
+                            var filename = parts[0];
+                            var anchor = parts.length > 1 ? '#' + parts[1] : '';
+                            node.setAttribute('href', '?' + gistId + '/' + filename + anchor);
+                        }
+                    }
+                }
+            });
+        });
     });
+
+    // Start observing once body exists
+    function startObserving() {
+        if (document.body) {
+            observer.observe(document.body, { childList: true, subtree: true });
+        } else {
+            setTimeout(startObserving, 10);
+        }
+    }
+    startObserving();
+
+    // Handle fragment navigation after dynamic content loads
+    // gisthost.github.io/gistpreview.github.io loads content dynamically, so the browser's
+    // native fragment navigation fails because the element doesn't exist yet
+    function scrollToFragment() {
+        var hash = window.location.hash;
+        if (!hash) return false;
+        var targetId = hash.substring(1);
+        var target = document.getElementById(targetId);
+        if (target) {
+            target.scrollIntoView({ behavior: 'smooth', block: 'start' });
+            return true;
+        }
+        return false;
+    }
+
+    // Try immediately in case content is already loaded
+    if (!scrollToFragment()) {
+        // Retry with increasing delays to handle dynamic content loading
+        var delays = [100, 300, 500, 1000, 2000];
+        delays.forEach(function(delay) {
+            setTimeout(scrollToFragment, delay);
+        });
+    }
 })();
 """
 
@@ -755,13 +1145,13 @@ def inject_gist_preview_js(output_dir):
     """Inject gist preview JavaScript into all HTML files in the output directory."""
     output_dir = Path(output_dir)
     for html_file in output_dir.glob("*.html"):
-        content = html_file.read_text()
+        content = html_file.read_text(encoding="utf-8")
         # Insert the gist preview JS before the closing </body> tag
         if "</body>" in content:
             content = content.replace(
                 "</body>", f"<script>{GIST_PREVIEW_JS}</script>\n</body>"
             )
-            html_file.write_text(content)
+            html_file.write_text(content, encoding="utf-8")
 
 
 def create_gist(output_dir, public=False):
@@ -849,9 +1239,10 @@ def generate_html(json_path, output_dir, github_repo=None):
         user_text = None
         if log_type == "user":
             content = message_data.get("content", "")
-            if isinstance(content, str) and content.strip():
+            text = extract_text_from_content(content)
+            if text:
                 is_user_prompt = True
-                user_text = content
+                user_text = text
         if is_user_prompt:
             if current_conv:
                 conversations.append(current_conv)
@@ -894,7 +1285,9 @@ def generate_html(json_path, output_dir, github_repo=None):
             pagination_html=pagination_html,
             messages_html="".join(messages_html),
         )
-        (output_dir / f"page-{page_num:03d}.html").write_text(page_content)
+        (output_dir / f"page-{page_num:03d}.html").write_text(
+            page_content, encoding="utf-8"
+        )
         print(f"Generated page-{page_num:03d}.html")
 
     # Calculate overall stats and collect all commits for timeline
@@ -977,7 +1370,7 @@ def generate_html(json_path, output_dir, github_repo=None):
         index_items_html="".join(index_items),
     )
     index_path = output_dir / "index.html"
-    index_path.write_text(index_content)
+    index_path.write_text(index_content, encoding="utf-8")
     print(
         f"Generated {index_path.resolve()} ({total_convs} prompts, {total_pages} pages)"
     )
@@ -1010,7 +1403,7 @@ def cli():
 @click.option(
     "--gist",
     is_flag=True,
-    help="Upload to GitHub Gist and output a gistpreview.github.io URL.",
+    help="Upload to GitHub Gist and output a gisthost.github.io URL.",
 )
 @click.option(
     "--json",
@@ -1031,8 +1424,6 @@ def cli():
 )
 def local_cmd(output, output_auto, repo, gist, include_json, open_browser, limit):
     """Select and convert a local Claude Code session to HTML."""
-    from datetime import datetime
-
     projects_folder = Path.home() / ".claude" / "projects"
 
     if not projects_folder.exists():
@@ -1100,7 +1491,7 @@ def local_cmd(output, output_auto, repo, gist, include_json, open_browser, limit
         inject_gist_preview_js(output)
         click.echo("Creating GitHub gist...")
         gist_id, gist_url = create_gist(output)
-        preview_url = f"https://gistpreview.github.io/?{gist_id}/index.html"
+        preview_url = f"https://gisthost.github.io/?{gist_id}/index.html"
         click.echo(f"Gist: {gist_url}")
         click.echo(f"Preview: {preview_url}")
 
@@ -1109,8 +1500,47 @@ def local_cmd(output, output_auto, repo, gist, include_json, open_browser, limit
         webbrowser.open(index_url)
 
 
+def is_url(path):
+    """Check if a path is a URL (starts with http:// or https://)."""
+    return path.startswith("http://") or path.startswith("https://")
+
+
+def fetch_url_to_tempfile(url):
+    """Fetch a URL and save to a temporary file.
+
+    Returns the Path to the temporary file.
+    Raises click.ClickException on network errors.
+    """
+    try:
+        response = httpx.get(url, timeout=60.0, follow_redirects=True)
+        response.raise_for_status()
+    except httpx.RequestError as e:
+        raise click.ClickException(f"Failed to fetch URL: {e}")
+    except httpx.HTTPStatusError as e:
+        raise click.ClickException(
+            f"Failed to fetch URL: {e.response.status_code} {e.response.reason_phrase}"
+        )
+
+    # Determine file extension from URL
+    url_path = url.split("?")[0]  # Remove query params
+    if url_path.endswith(".jsonl"):
+        suffix = ".jsonl"
+    elif url_path.endswith(".json"):
+        suffix = ".json"
+    else:
+        suffix = ".jsonl"  # Default to JSONL
+
+    # Extract a name from the URL for the temp file
+    url_name = Path(url_path).stem or "session"
+
+    temp_dir = Path(tempfile.gettempdir())
+    temp_file = temp_dir / f"claude-url-{url_name}{suffix}"
+    temp_file.write_text(response.text, encoding="utf-8")
+    return temp_file
+
+
 @cli.command("json")
-@click.argument("json_file", type=click.Path(exists=True))
+@click.argument("json_file", type=click.Path())
 @click.option(
     "-o",
     "--output",
@@ -1130,7 +1560,7 @@ def local_cmd(output, output_auto, repo, gist, include_json, open_browser, limit
 @click.option(
     "--gist",
     is_flag=True,
-    help="Upload to GitHub Gist and output a gistpreview.github.io URL.",
+    help="Upload to GitHub Gist and output a gisthost.github.io URL.",
 )
 @click.option(
     "--json",
@@ -1145,19 +1575,36 @@ def local_cmd(output, output_auto, repo, gist, include_json, open_browser, limit
     help="Open the generated index.html in your default browser (default if no -o specified).",
 )
 def json_cmd(json_file, output, output_auto, repo, gist, include_json, open_browser):
-    """Convert a Claude Code session JSON/JSONL file to HTML."""
+    """Convert a Claude Code session JSON/JSONL file or URL to HTML."""
+    # Handle URL input
+    if is_url(json_file):
+        click.echo(f"Fetching {json_file}...")
+        temp_file = fetch_url_to_tempfile(json_file)
+        json_file_path = temp_file
+        # Use URL path for naming
+        url_name = Path(json_file.split("?")[0]).stem or "session"
+    else:
+        # Validate that local file exists
+        json_file_path = Path(json_file)
+        if not json_file_path.exists():
+            raise click.ClickException(f"File not found: {json_file}")
+        url_name = None
+
     # Determine output directory and whether to open browser
     # If no -o specified, use temp dir and open browser by default
     auto_open = output is None and not gist and not output_auto
     if output_auto:
         # Use -o as parent dir (or current dir), with auto-named subdirectory
         parent_dir = Path(output) if output else Path(".")
-        output = parent_dir / Path(json_file).stem
+        output = parent_dir / (url_name or json_file_path.stem)
     elif output is None:
-        output = Path(tempfile.gettempdir()) / f"claude-session-{Path(json_file).stem}"
+        output = (
+            Path(tempfile.gettempdir())
+            / f"claude-session-{url_name or json_file_path.stem}"
+        )
 
     output = Path(output)
-    generate_html(json_file, output, github_repo=repo)
+    generate_html(json_file_path, output, github_repo=repo)
 
     # Show output directory
     click.echo(f"Output: {output.resolve()}")
@@ -1165,9 +1612,8 @@ def json_cmd(json_file, output, output_auto, repo, gist, include_json, open_brow
     # Copy JSON file to output directory if requested
     if include_json:
         output.mkdir(exist_ok=True)
-        json_source = Path(json_file)
-        json_dest = output / json_source.name
-        shutil.copy(json_file, json_dest)
+        json_dest = output / json_file_path.name
+        shutil.copy(json_file_path, json_dest)
         json_size_kb = json_dest.stat().st_size / 1024
         click.echo(f"JSON: {json_dest} ({json_size_kb:.1f} KB)")
 
@@ -1176,7 +1622,7 @@ def json_cmd(json_file, output, output_auto, repo, gist, include_json, open_brow
         inject_gist_preview_js(output)
         click.echo("Creating GitHub gist...")
         gist_id, gist_url = create_gist(output)
-        preview_url = f"https://gistpreview.github.io/?{gist_id}/index.html"
+        preview_url = f"https://gisthost.github.io/?{gist_id}/index.html"
         click.echo(f"Gist: {gist_url}")
         click.echo(f"Preview: {preview_url}")
 
@@ -1263,9 +1709,10 @@ def generate_html_from_session_data(session_data, output_dir, github_repo=None):
         user_text = None
         if log_type == "user":
             content = message_data.get("content", "")
-            if isinstance(content, str) and content.strip():
+            text = extract_text_from_content(content)
+            if text:
                 is_user_prompt = True
-                user_text = content
+                user_text = text
         if is_user_prompt:
             if current_conv:
                 conversations.append(current_conv)
@@ -1308,7 +1755,9 @@ def generate_html_from_session_data(session_data, output_dir, github_repo=None):
             pagination_html=pagination_html,
             messages_html="".join(messages_html),
         )
-        (output_dir / f"page-{page_num:03d}.html").write_text(page_content)
+        (output_dir / f"page-{page_num:03d}.html").write_text(
+            page_content, encoding="utf-8"
+        )
         click.echo(f"Generated page-{page_num:03d}.html")
 
     # Calculate overall stats and collect all commits for timeline
@@ -1391,7 +1840,7 @@ def generate_html_from_session_data(session_data, output_dir, github_repo=None):
         index_items_html="".join(index_items),
     )
     index_path = output_dir / "index.html"
-    index_path.write_text(index_content)
+    index_path.write_text(index_content, encoding="utf-8")
     click.echo(
         f"Generated {index_path.resolve()} ({total_convs} prompts, {total_pages} pages)"
     )
@@ -1422,7 +1871,7 @@ def generate_html_from_session_data(session_data, output_dir, github_repo=None):
 @click.option(
     "--gist",
     is_flag=True,
-    help="Upload to GitHub Gist and output a gistpreview.github.io URL.",
+    help="Upload to GitHub Gist and output a gisthost.github.io URL.",
 )
 @click.option(
     "--json",
@@ -1536,7 +1985,7 @@ def web_cmd(
         inject_gist_preview_js(output)
         click.echo("Creating GitHub gist...")
         gist_id, gist_url = create_gist(output)
-        preview_url = f"https://gistpreview.github.io/?{gist_id}/index.html"
+        preview_url = f"https://gisthost.github.io/?{gist_id}/index.html"
         click.echo(f"Gist: {gist_url}")
         click.echo(f"Preview: {preview_url}")
 
@@ -1545,5 +1994,129 @@ def web_cmd(
         webbrowser.open(index_url)
 
 
+@cli.command("all")
+@click.option(
+    "-s",
+    "--source",
+    type=click.Path(exists=True),
+    help="Source directory containing Claude projects (default: ~/.claude/projects).",
+)
+@click.option(
+    "-o",
+    "--output",
+    type=click.Path(),
+    default="./claude-archive",
+    help="Output directory for the archive (default: ./claude-archive).",
+)
+@click.option(
+    "--include-agents",
+    is_flag=True,
+    help="Include agent-* session files (excluded by default).",
+)
+@click.option(
+    "--dry-run",
+    is_flag=True,
+    help="Show what would be converted without creating files.",
+)
+@click.option(
+    "--open",
+    "open_browser",
+    is_flag=True,
+    help="Open the generated archive in your default browser.",
+)
+@click.option(
+    "-q",
+    "--quiet",
+    is_flag=True,
+    help="Suppress all output except errors.",
+)
+def all_cmd(source, output, include_agents, dry_run, open_browser, quiet):
+    """Convert all local Claude Code sessions to a browsable HTML archive.
+
+    Creates a directory structure with:
+    - Master index listing all projects
+    - Per-project pages listing sessions
+    - Individual session transcripts
+    """
+    # Default source folder
+    if source is None:
+        source = Path.home() / ".claude" / "projects"
+    else:
+        source = Path(source)
+
+    if not source.exists():
+        raise click.ClickException(f"Source directory not found: {source}")
+
+    output = Path(output)
+
+    if not quiet:
+        click.echo(f"Scanning {source}...")
+
+    projects = find_all_sessions(source, include_agents=include_agents)
+
+    if not projects:
+        if not quiet:
+            click.echo("No sessions found.")
+        return
+
+    # Calculate totals
+    total_sessions = sum(len(p["sessions"]) for p in projects)
+
+    if not quiet:
+        click.echo(f"Found {len(projects)} projects with {total_sessions} sessions")
+
+    if dry_run:
+        # Dry-run always outputs (it's the point of dry-run), but respects --quiet
+        if not quiet:
+            click.echo("\nDry run - would convert:")
+            for project in projects:
+                click.echo(
+                    f"\n  {project['name']} ({len(project['sessions'])} sessions)"
+                )
+                for session in project["sessions"][:3]:  # Show first 3
+                    mod_time = datetime.fromtimestamp(session["mtime"])
+                    click.echo(
+                        f"    - {session['path'].stem} ({mod_time.strftime('%Y-%m-%d')})"
+                    )
+                if len(project["sessions"]) > 3:
+                    click.echo(f"    ... and {len(project['sessions']) - 3} more")
+        return
+
+    if not quiet:
+        click.echo(f"\nGenerating archive in {output}...")
+
+    # Progress callback for non-quiet mode
+    def on_progress(project_name, session_name, current, total):
+        if not quiet and current % 10 == 0:
+            click.echo(f"  Processed {current}/{total} sessions...")
+
+    # Generate the archive using the library function
+    stats = generate_batch_html(
+        source,
+        output,
+        include_agents=include_agents,
+        progress_callback=on_progress,
+    )
+
+    # Report any failures
+    if stats["failed_sessions"]:
+        click.echo(f"\nWarning: {len(stats['failed_sessions'])} session(s) failed:")
+        for failure in stats["failed_sessions"]:
+            click.echo(
+                f"  {failure['project']}/{failure['session']}: {failure['error']}"
+            )
+
+    if not quiet:
+        click.echo(
+            f"\nGenerated archive with {stats['total_projects']} projects, "
+            f"{stats['total_sessions']} sessions"
+        )
+        click.echo(f"Output: {output.resolve()}")
+
+    if open_browser:
+        index_url = (output / "index.html").resolve().as_uri()
+        webbrowser.open(index_url)
+
+
 def main():
     cli()
